{
===============================================================================================
Module for inclustion of sopurt for MySQL database in Delphi 
components and applications.
Copyright (C) 1999 Mikael Ostensson (mikaelo@images.com)

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Library General Public
License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Library General Public License for more details.

You should have received a copy of the GNU Library General Public
License along with this library; if not, write to the
Free Software Foundation, Inc., 59 Temple Place - Suite 330,
Boston, MA  02111-1307, USA.
===============================================================================================
}

{$I VG.INC}
{$D-,L-}

unit vgMySQLIntf;

interface

uses Windows, Winsock;

const
 DEFAULT_DLL_LOCATION   = 'libmySQL.DLL';

{General Declarations}
  MYSQL_ERRMSG_SIZE      = 200;
  MYSQL_PORT             = 3306;
  LOCAL_HOST             = 'localhost';
  NAME_LEN               = 64;
  PROTOCOL_VERSION       = 10;
  FRM_VER                = 6;

{Enum Field Types}
  FIELD_TYPE_DECIMAL     = 0;
  FIELD_TYPE_TINY        = 1;
  FIELD_TYPE_SHORT       = 2;
  FIELD_TYPE_LONG        = 3;
  FIELD_TYPE_FLOAT       = 4;
  FIELD_TYPE_DOUBLE      = 5;
  FIELD_TYPE_NULL        = 6;
  FIELD_TYPE_TIMESTAMP   = 7;
  FIELD_TYPE_LONGLONG    = 8;
  FIELD_TYPE_INT24       = 9;
  FIELD_TYPE_DATE        = 10;
  FIELD_TYPE_TIME        = 11;
  FIELD_TYPE_DATETIME    = 12;
  FIELD_TYPE_YEAR        = 13;
  FIELD_TYPE_NEWDATE     = 14;
  FIELD_TYPE_ENUM        = 247;
  FIELD_TYPE_SET         = 248;
  FIELD_TYPE_TINY_BLOB   = 249;
  FIELD_TYPE_MEDIUM_BLOB = 250;
  FIELD_TYPE_LONG_BLOB   = 251;
  FIELD_TYPE_BLOB        = 252;
  FIELD_TYPE_VAR_STRING  = 253;
  FIELD_TYPE_STRING      = 254;

{For Compatibility}
  FIELD_TYPE_CHAR         = FIELD_TYPE_TINY;
  FIELD_TYPE_INTERVAL     = FIELD_TYPE_ENUM;

{Server Administration Refresh Options}
  _REFRESH_GRANT	 = 1;     //Refresh grant tables
  _REFRESH_LOG		 = 2;     // Start on new log file
  _REFRESH_TABLES	 = 4;     // close all tables
  _REFRESH_HOSTS	 = 8;     // Flush host cache
  _REFRESH_FAST		 = 32768; // Intern flag

// Client Connection Options
  _CLIENT_LONG_PASSWORD	 = 1;	// new more secure passwords
  _CLIENT_FOUND_ROWS	 = 2;	// Found instead of affected rows
  _CLIENT_LONG_FLAG	 = 4;	// Get all column flags
  _CLIENT_CONNECT_WITH_DB = 8;	// One can specify db on connect
  _CLIENT_NO_SCHEMA	 = 16;	// Don't allow database.table.column
  _CLIENT_COMPRESS	 = 32;	// Can use compression protcol
  _CLIENT_ODBC		 = 64;	// Odbc client
  _CLIENT_LOCAL_FILES	 = 128;	// Can use LOAD DATA LOCAL
  _CLIENT_IGNORE_SPACE	 = 256;	// Ignore spaces before '('

type
  //The DataType my_ulonglong is defined in the dll as _int64
  //Due to the lack of a 64 bit data type I created this structure to
  //pad out the additional 32  bits
  //this limmits the resultsets to have 2147483647 rows.
  //However this limit can not  be met anyway due to the fact that
  //internal implementation of the TMyDataset Uses a Single TList To
  //store all fields in and thus can have a maximun of 2147483647 fields and any matrix
  //Imight redo this implementation too store each record instead but to have a somewhat easy
  //access to any give field that would add an additional 4 bytes to each record.

  Tmy_ulonglong = record
                Data:Longint;
                pad :Longint;
  end;

  //if you are using Delphi 4+ you may want to declare is in one of the following ways:
  //For code compatibility with D3:
  //Tmy_ulonglong = record
  //   Data: Int64;
  //end;
  //
  //or for a cleaner look:
  //Tmy_ulonglong = Int64;


  TClientCapabilities = (
    CLIENT_LONG_PASSWORD,
    CLIENT_FOUND_ROWS,
    CLIENT_LONG_FLAG,
    CLIENT_CONNECT_WITH_DB,
    CLIENT_NO_SCHEMA,
    CLIENT_COMPRESS,
    CLIENT_ODBC,
    CLIENT_LOCAL_FILES,
    CLIENT_IGNORE_SPACE
  );

  TSetClientCapabilities = set of TClientCapabilities;

  TRefreshOptions = (
    REFRESH_GRANT,
    REFRESH_LOG,
    REFRESH_TABLES,
    REFRESH_HOSTS,
    REFRESH_FAST
  );
  TSetRefreshOptions = set of TRefreshOptions;

  _mysql_status = (
    MYSQL_STATUS_READY,
    MYSQL_STATUS_GET_RESULT,
    MYSQL_STATUS_USE_RESULT
  );

  _mysql_option = (
    MYSQL_OPT_CONNECT_TIMEOUT,
    MYSQL_OPT_COMPRESS
  );

  ENUM_FIELD_TYPES = byte;
  MY_BOOL          = char;
  GPTR             = pchar;
  SOCKET           = word;
  pCardinal        = ^Cardinal;
  pLong            = ^LongInt;

  PUSED_MEM=^USED_MEM;
  USED_MEM = packed record
    next  : pused_mem;
    left,
    size  : word;
  end;

  PERR_PROC = ^ERR_PROC;
  ERR_PROC = procedure;

  PMEM_ROOT = ^MEM_ROOT;
  MEM_ROOT = packed record
    free,
    used          : pused_mem;
    min_malloc,
    block_size    : word;
    error_handler : PERR_PROC;
  end;

  NET = packed record
    nettype                 : integer;//ENUM_FIELD_TYPES;
    hPipe                   : integer; // if Windows
    fd                      : integer;
    fcntl                   : integer;
    buff,
    buff_end,
    write_pos, read_pos     : Pchar;
    last_error              : array[01..MYSQL_ERRMSG_SIZE] of char;
    last_errno,
    max_packet,
    timeout,
    pkt_nr                  : integer; //Cardinal;
    error,
    return_errno,
    compress                : smallint;
    remain_in_buf_,length,
    buf_length,
    where_b                 : longint;
    more                    :my_bool;
    save_char               : char;
  end;



PMYSQL_FIELD=^MYSQL_FIELD;
MYSQL_FIELD=record
    name,
    table,
    def          : pchar;
    _type        : ENUM_FIELD_TYPES;
    length,
    max_length,
    flags,
    decimals     : Cardinal;
end;
MYSQL_FIELD_OFFSET= Cardinal;

PMYSQL_ROW = ^MYSQL_ROW;
MYSQL_ROW= array[00..$ff] of pchar;


PMYSQL_ROWS = ^MYSQL_ROWS;
MYSQL_ROWS = record
    next : PMYSQL_ROWS;
    data : PMYSQL_ROW;
end;

MYSQL_ROW_OFFSET = PMYSQL_ROWS;

PMYSQL_DATA = ^MYSQL_DATA;
MYSQL_DATA = record
  rows     : Tmy_ulonglong;
  fields   : Cardinal;
  data     : PMYSQL_ROWS;
  alloc    : MEM_ROOT;
end;



type
  PMYSQL_OPTIONS = ^_MYSQL_OPTIONS;
  _MYSQL_OPTIONS = record
    connect_timeout,
    clientFlag      : integer;
    compress,
    named_pipe      : MY_BOOL;
    port: integer;
    host,
    init_coppand,
    user,
    password,
    unix_socket,
    db,
    my_cnf,
    my_cnf_group    : pchar;
end;




PMYSQL = ^MYSQL;
MYSQL=  record
      _net                  : NET;
      host,
      user,
      passwd,
      unix_socket,
      server_version,
      host_info,
      info,
      db                    : pchar;
      port,
      client_flag,
      server_capabilities,
      protocol_version,
      field_count           : integer;
      thread_id             : cardinal;
      affected_rows,
      insert_id,
      extra_info            : Tmy_ulonglong;
      packet_length         : longint;
      status                : _mysql_status;
      fields                : PMYSQL_FIELD;
      field_alloc           : MEM_ROOT;
      free_me, reconnect    : MY_BOOL;
      options               : _mysql_options;

      reserved             : array[0..10] of Byte;
end;



PMYSQL_RES = ^MYSQL_RES;
MYSQL_RES =  packed record
  row_count                  : Tmy_ulonglong;
  field_count,
  current_field              : word; // longint
  fields                     : PMYSQL_FIELD;
  data                       : PMYSQL_DATA;
  data_cursor                : PMYSQL_ROWS;
  field_alloc                : MEM_ROOT;
  row,
  current_row                : PMYSQL_ROW;
  lengths                    : ^Cardinal;
  handle                     : ^MYSQL;
  eof                        : MY_BOOL;
end;


type
{
  TModifyType = (MODIFY_INSERT, MODIFY_UPDATE, MODIFY_DELETE);
  TQuoteOptions = (QUOTE_STRIP_CR,QUOTE_STRIP_LF);
  TQuoteOptionsSet = set of TQuoteOptions;
}
  Tmysql_debug =           procedure (debug: pchar);
  Tmysql_dump_debug_info = function  (_mysql: pmysql) : integer;
  Tmysql_init =            function  (_mysql: pmysql) : pmysql; stdcall;
  Tmysql_connect =         function  (_mysql: pmysql; const host, user, passwd: pchar) : pmysql; stdcall;
  Tmysql_real_connect =    function  (_mysql: pmysql; const host, user, passwd, db: pchar; port:Cardinal; unix_socket: PChar; clientflag: Cardinal) : pmysql; stdcall;
  Tmysql_close =           procedure (_mysql: pmysql); stdcall;
  Tmysql_query =           function  (_mysql: pmysql; const query: pchar) : integer; stdcall;
  Tmysql_real_query =      function  (_mysql: pmysql; const query: pchar; len: integer) : integer; stdcall;
  Tmysql_select_db =       function  (_mysql: pmysql; const DB: pchar) : integer; stdcall;
  Tmysql_create_db =       function  (_mysql: pmysql; const DB: PChar) : integer; stdcall;
  Tmysql_drop_db =         function  (_mysql: pmysql; const DB: PChar) : integer; stdcall;
  Tmysql_shutdown =        function  (_mysql: pmysql): Integer; stdcall;
  Tmysql_refresh =         function  (_mysql: pmysql; refresh_options: Cardinal) : integer; stdcall;
  Tmysql_kill =            function  (_mysql: pmysql; pid: longint) : integer; stdcall;
  Tmysql_ping =            function  (_mysql: pmysql) : integer; stdcall;
  Tmysql_stat =            function  (_mysql: pmysql) : pchar; stdcall;
  Tmysql_options =         function  (_mysql: pmysql; _option: _mysql_options; const arg: PChar) : Integer;stdcall;
  Tmysql_escape_string =   function  (pto, pfrom: pchar; len: Cardinal) : Cardinal;stdcall;
  Tmysql_get_server_info = function  (_mysql: pmysql) : PChar; stdcall;
  Tmysql_get_client_info = function : PChar; stdcall;
  Tmysql_get_host_info =   function  (_mysql: pmysql) : PChar; stdcall;
  Tmysql_get_proto_info =  function  (_mysql: pmysql) : Cardinal; stdcall;
  Tmysql_list_dbs =        function  (_mysql: pmysql; wild: pchar) : pmysql_res; stdcall;
  Tmysql_list_tables =     function  (_mysql: pmysql; const wild: pchar) : pmysql_res; stdcall;
  Tmysql_list_fields =     function  (_mysql: pmysql; const table, wild: pchar) : pmysql_res; stdcall;
  Tmysql_list_processes =  function  (_mysql: pmysql) : pmysql_res; stdcall;
  Tmysql_store_result =    function  (_mysql: pmysql) : pmysql_res; stdcall;
  Tmysql_use_result =      function  (_mysql: pmysql) : pmysql_res; stdcall;
  Tmysql_free_result =     procedure (result: pmysql_res); stdcall;
  Tmysql_fetch_row =       function  (result: pmysql_res) : pmysql_row; stdcall;
  Tmysql_fetch_lengths =   function  (result: pmysql_res) : pLong; stdcall;
  Tmysql_fetch_field =     function  (result: pmysql_res) : pmysql_field; stdcall;
  Tmysql_data_seek =       procedure (result: pmysql_res; offset: Cardinal); stdcall;
  Tmysql_row_seek =        function  (result: pmysql_res; row: mysql_row_offset) : mysql_row_offset; stdcall;
  Tmysql_field_seek =      function  (result: pmysql_res; offset: mysql_field_offset) : mysql_field_offset; stdcall;


  function MySqlLoadLib : boolean;
  procedure MySqlUnloadLib;

const
  DLL : string = DEFAULT_DLL_LOCATION;

var
  mysql_debug           : Tmysql_debug;
  mysql_dump_debug_info : Tmysql_dump_debug_info;
  mysql_init            : Tmysql_init;
  mysql_connect         : Tmysql_connect;
  mysql_real_connect    : Tmysql_real_connect;
  mysql_close           : Tmysql_close;
  mysql_select_db       : Tmysql_select_db;
  mysql_create_db       : Tmysql_create_db;
  mysql_drop_db         : Tmysql_drop_db;
  mysql_query           : Tmysql_query;
  mysql_real_query      : Tmysql_real_query;
  mysql_shutdown        : Tmysql_shutdown;
  mysql_refresh         : Tmysql_refresh;
  mysql_kill            : Tmysql_kill;
  mysql_ping            : Tmysql_ping;
  mysql_stat            : Tmysql_stat;
  mysql_options         : Tmysql_options;
  mysql_escape_string   : Tmysql_escape_string;
  mysql_get_server_info : Tmysql_get_server_info;
  mysql_get_client_info : Tmysql_get_client_info;
  mysql_get_host_info   : Tmysql_get_host_info;
  mysql_get_proto_info  : Tmysql_get_proto_info;
  mysql_list_dbs        : Tmysql_list_dbs;
  mysql_list_tables     : Tmysql_list_tables;
  mysql_list_fields     : Tmysql_list_fields;
  mysql_list_processes  : Tmysql_list_processes;
  mysql_data_seek       : Tmysql_data_seek;
  mysql_row_seek        : Tmysql_row_seek;
  mysql_field_seek      : Tmysql_field_seek;
  mysql_fetch_row       : Tmysql_fetch_row;
  mysql_fetch_lengths   : Tmysql_fetch_lengths;
  mysql_fetch_field     : Tmysql_fetch_field;
  mysql_store_result    : Tmysql_store_result;
  mysql_use_result      : Tmysql_use_result;
  mysql_free_result     : Tmysql_free_result;


implementation

var
  hDLL: THandle;
  LoadCount: Integer;

function MySqlLoadLib: boolean;
begin
  if hDLL <> 0 then
  begin
    Inc(LoadCount);
    Result := True;
    Exit;
  end;

  hDLL := LoadLibrary(PChar(DLL));

  Result := hDll <> 0;

  if Result then
  begin
    Inc(LoadCount);
    @mysql_debug           := GetProcAddress(hDLL,'mysql_debug');
    @mysql_dump_debug_info := GetProcAddress(hDLL,'mysql_dump_debug_info');
    @mysql_init            := GetProcAddress(hDLL,'mysql_init');
    @mysql_connect         := GetProcAddress(hDLL,'mysql_connect');
    @mysql_real_connect    := GetProcAddress(hDLL,'mysql_real_connect');
    @mysql_close           := GetProcAddress(hDLL,'mysql_close');
    @mysql_select_db       := GetProcAddress(hDLL,'mysql_select_db');
    @mysql_create_db       := GetProcAddress(hDLL,'mysql_create_db');
    @mysql_drop_db         := GetProcAddress(hDLL,'mysql_drop_db');
    @mysql_query           := GetProcAddress(hDLL,'mysql_query');
    @mysql_real_query      := GetProcAddress(hDLL,'mysql_real_query');
    @mysql_shutdown        := GetProcAddress(hDLL,'mysql_shutdown');
    @mysql_refresh         := GetProcAddress(hDLL,'mysql_refresh');
    @mysql_kill            := GetProcAddress(hDLL,'mysql_kill');
    @mysql_ping            := GetProcAddress(hDLL,'mysql_ping');
    @mysql_stat            := GetProcAddress(hDLL,'mysql_stat');
    @mysql_options         := GetProcAddress(hDLL,'mysql_options');
    @mysql_escape_string   := GetProcAddress(hDLL,'mysql_escape_string');
    @mysql_get_server_info := GetProcAddress(hDLL,'mysql_get_server_info');
    @mysql_get_client_info := GetProcAddress(hDLL,'mysql_get_client_info');
    @mysql_get_host_info   := GetProcAddress(hDLL,'mysql_get_host_info');
    @mysql_get_proto_info  := GetProcAddress(hDLL,'mysql_get_proto_info');
    @mysql_list_fields     := GetProcAddress(hDLL,'mysql_list_fields');
    @mysql_list_processes  := GetProcAddress(hDLL,'mysql_list_processes');
    @mysql_list_dbs        := GetProcAddress(hDLL,'mysql_list_dbs');
    @mysql_list_tables     := GetProcAddress(hDLL,'mysql_list_tables');
    @mysql_data_seek       := GetProcAddress(hDLL,'mysql_data_seek');
    @mysql_row_seek        := GetProcAddress(hDLL,'mysql_row_seek');
    @mysql_field_seek      := GetProcAddress(hDLL,'mysql_field_seek');
    @mysql_fetch_row       := GetProcAddress(hDLL,'mysql_fetch_row');
    @mysql_fetch_lengths   := GetProcAddress(hDLL,'mysql_fetch_lengths');
    @mysql_fetch_field     := GetProcAddress(hDLL,'mysql_fetch_field');
    @mysql_use_result      := GetProcAddress(hDLL,'mysql_use_result');
    @mysql_store_result    := GetProcAddress(hDLL,'mysql_store_result');
    @mysql_free_result     := GetProcAddress(hDLL,'mysql_free_result');
  end;
end;

procedure MySqlUnloadLib;
begin
  if hDLL <> 0 then
  begin
    Dec(LoadCount);
    if LoadCount = 0 then
    begin
      FreeLibrary(hDLL);
      hDLL := 0;
    end;
  end;
end;

end.
